#include ".\BasicBall.h"

// BasicBall Class implementation


BasicBall::BasicBall() : PhysicsObject()
{
	ColorInfo temp;
	temp.mColor = 0;
	temp.mImgNum = 0;
	
	mColor = temp;
	mCanPassFront = mCanPassBack = mCanPassPreview = mCanPassShoot = mPreviewBall = false;
}

BasicBall::BasicBall( NewtonWorld* world,  ColorInfo color, dVector pos, int newtonMat, int objID ) : PhysicsObject()
{
	ColorInfo temp;
	temp.mColor = 0;
	temp.mImgNum = 0;
	
	mColor = temp;
	mCanPassFront = mCanPassBack = mCanPassPreview = mCanPassShoot = mPreviewBall = false;

	init( world, color, pos, newtonMat, objID );
}


BasicBall::~BasicBall()
{
}


void BasicBall::init( NewtonWorld* world, ColorInfo color, dVector pos, int newtonMat, int objID )
{
	// create a BasicBall, and set it at a start position.
	mWorld = world;

	mObjNum = Helpers::getOpenObjNum();

	dbCloneObject( mObjNum, objID );
	dbScaleObject( mObjNum, 51.0, 51.0f, 51.0f );

	// set color!
	mColor = color;
	dbColorObject( mObjNum, mColor.mColor );
	dbTextureObject( mObjNum, mColor.mImgNum );
	

	// make the rigid body.
	NewtonCollision* col = NewtonCreateSphere( world, 0.51f, 0.51f, 0.51f, NULL );
	mBody = NewtonCreateBody( world, col );
	NewtonReleaseCollision( world, col );

	// set the newton material.
	NewtonBodySetMaterialGroupID( mBody, newtonMat );

	// set the type
	setType( BT_BALL );

	// setup basic physics stuff.
	_doPhysicsWiring();

	// set mass.
	float mass = 10.0;
	dVector inertia = Helpers::CalcSphereSolid( mass, 1.0 );
	setMassMatrix(mass, inertia );



	// initial position.
	dVector angle(0.0f, 0.0f, 0.0f);
	setPosition( pos );
	setEulerAngles( angle );

}


int BasicBall::possibleTouch( BasicBall* ball )
{
	// this ball is the same color as us.  should we add it to our map?
	BallContactMapIterator i = mBallsInContact.find( ball->mObjNum );

	if (i == mBallsInContact.end())
	{
		// not found, so add it!
		mBallsInContact[ball->mObjNum] = ball;
	}

	return (int)mBallsInContact.size();
}




void _cdecl BasicBall::_BasicBallClearCallback( const NewtonBody* body )
{
	// get the physics object.
	PhysicsObject* obj = (PhysicsObject*)NewtonBodyGetUserData( body );

	if (obj->getType() == BT_BALL)
	{
		//this is a ball, destroy it!
		BasicBall* ball = (BasicBall*)obj;

		// clear the map.
		ball->mBallsInContact.clear();

		// also clear the # of balls hit.
		ball->mBallsHit = 0;

		// add me!
		ball->mBallsInContact[ ball->mObjNum ] = ball;
	}
}
	
