// --------------------------------
#include <DarkSDK.h>
#include <Newton.h>

// --------------------------------
// includes for this project.
#include "BasicBall.h"
#include "BallFrame.h"
#include "UserCrosshair.h"
#include "SkySphere.h"
#include "SFXManager.h"
#include "SFX_KillBall.h"
#include "SFX_Smoke.h"
#include "SFX_Bomb.h"
#include "ScoreBoard.h"
#include "OverflowChecker.h"
#include "WorldInfo.h"
#include "MenuSystem.h"
#include "SoundManager.h"

// --------------------------------
// structures...

struct CollisionData
{
	NewtonBody* mBody0;
	NewtonBody* mBody1;

	BasicBall* mNextBall;
	BasicBall* mShootBall;

	dVector mShootPosition;

	std::vector<BasicBall*> mBallsToCheck;

	bool mHitSound;
	bool mBombSound;
	bool mWildSound;
};




// --------------------------------
// function declarations...
void loadHighScores();
void saveHighScores();
void updateHighScores();

void getASCII( int scancode, char* ascii );

void setupFader( int color );
void fadeOut( float fadeTimeInSec );
void fadeIn( float fadeTimeInSec );

void OneGameLoop( const float deltat, CollisionData& coldata );

// setup the spout locations.
void setupSpouts();

// create a new ball at one of the spouts.
void spawnNewBall();

// create a new ball for the user to throw.
void spawnNewPreviewBall();

// recursive ball deleter
void RecursiveCheck( BasicBall* checkme, BasicBall::BallContactMap& toDelete, BasicBall::BallContactMap& checked );


int scrollHiScores( int toprow, ScoreBoard* board, std::vector<char*>& textvector );

// setup the main game colors...
void setupColors();

// delete colors.
void deleteColors();

// delete global stuff
void deleteGlobalResources();


// get color info from enumeration.
ColorInfo getColorInfo( Colors color );

// "memory-leak" test.
void checkForLeaks();

// --------------------------------------
// newton callback for wild-balls
void _cdecl newtonWildCallback( const NewtonBody* body );

// newton callback for  bomb!
void _cdecl newtonBombCallback( const NewtonBody* body );

// leave world callback.
void _cdecl newtonLeaveWorld( const NewtonBody* body );

// callback to destroy all existing balls.
void _cdecl newtonDestroyBalls( const NewtonBody* body );

// material callback functions...
int _cdecl newtonContactBegin(const NewtonMaterial* material, const NewtonBody* body0, const NewtonBody* body1 );
int _cdecl newtonContactProcess( const NewtonMaterial* material, const NewtonContact* contact );
void _cdecl newtonContactEnd( const NewtonMaterial* material );