#include "OverflowChecker.h"


OverflowChecker::OverflowChecker( NewtonWorld* world, dVector start, dVector end, float timeToSetOff )
{
	mWorld = world;
	mStart = start;
	mEnd = end;
	mTimeToSetOff = timeToSetOff;
	mTimeTripped = 0.0f;

	dVector to = end - start;
	mGoalLength = Helpers::dVectorLength( to );

	// first make a simple box object to represent the raypath.
	mObjNum = Helpers::getOpenObjNum();

	dbMakeObjectBox( mObjNum, 1.0f, 0.1f, 0.1f );
	dbColorObject( mObjNum, dbRgb(255,0,0) );

	dbOffsetLimb( mObjNum, 0, 0.5f, 0.0f, 0.0f );

	dbRotateObject( mObjNum, 0, -90, 0 );
	dbFixObjectPivot( mObjNum );

	dbPositionObject( mObjNum, start.m_x, start.m_y, start.m_z );
	dbPointObject( mObjNum, end.m_x, end.m_y, end.m_z );

}


OverflowChecker::~OverflowChecker()
{
	dbDeleteObject( mObjNum );
}


int OverflowChecker::update( float timeStep )
{
	// perform a raycast.
	mRayDist = 1.0f;

	NewtonWorldRayCast( mWorld, &mStart.m_x, &mEnd.m_x, OverflowChecker::newtonRaycastCallback, this );


	// update the visual object.
	float actuallength = (mGoalLength * mRayDist);
	dbScaleObject( mObjNum, actuallength * 100.0f, 100.0f, 100.0f );
	
	
	if (mRayDist < 1.0f)
	{
		mTimeTripped += timeStep;

		if (mTimeTripped >= mTimeToSetOff)
			return 1;
	}
	else
	{
		mTimeTripped = 0.0f;
	}

	return 0;
}

float _cdecl OverflowChecker::newtonRaycastCallback(const NewtonBody* body, const float* hitNormal, int collisionID, void* userData, float intersetParam)
{
	// we have hit some kind of body.
	OverflowChecker* me = (OverflowChecker*)userData;

	me->mRayDist = intersetParam;

	return 1.1f;
}



