/*
	ConnectMore project

	by Walaber

	CLASS: PhysicsObject - simple class inhrited by all objects with physics properties.
*/

#ifndef _PHYSICSOBJECT_INCLUDED_
#define _PHYSICSOBJECT_INCLUDED_

#include "Prereqs.h"

#include "3dmath.h"
#include <Newton.h>
#include <DarkSDK.h>

class PhysicsObject
{
public:
	PhysicsObject();
	~PhysicsObject();

	// ------------------------------
	// INTERNAL USE FUNCTIONS
	// ------------------------------
	void _doPhysicsWiring();


	// ------------------------------
	// COMMON FUNCTIONS
	// ------------------------------
	// get the type
	int getType() { return mType; }

	//set the type
	void setType( int type ) { mType = type; }

	// get global position
	dVector getPosition();

	// set global position
	void setPosition( dVector& pos );

	// get global position and orientation as a 4x4 matrix
	dMatrix getMatrix();

	// set position and orientation from a 4x4 matrix.
	void setMatrix( dMatrix& mat );

	// get rotation as Euler angles (in degrees)
	dVector getEulerAngles();

	// set rotation as Euler angles (in degrees)
	void setEulerAngles( dVector& angles );

	// get velocity
	dVector getVelocity();

	// set velocity
	void setVelocity( dVector& vel );

	// get the mass.
	float getMass();

	// get mass and inertia
	void getMassMatrix( float& mass, dVector& inertia );

	// set mass and inertia
	void setMassMatrix( float& mass, dVector& inertia );

	// freeze
	void freeze();

	// unfreeze
	void unfreeze();

	// ------------------------
	// Newton world
	NewtonWorld* mWorld;

	// rigid body
	NewtonBody* mBody;

	// object number
	int mObjNum;

	// Type ID used for collision callbacks.
	int mType;


private:
	void _updateVisualObjectMatrix( const float* matrix );

	// newton callbacks.
	static void _cdecl _newtonTransformCallback( const NewtonBody* body, const float* matrix );
	static void _cdecl _newtonForceCallback( const NewtonBody* body );

};

#endif // _PHYSICSOBJECT_INCLUDED_