/*
	ConnectMore project

	by Walaber

	CLASS: SFXManager / SFXObject - the main system for creating special effect objects.
*/

#ifndef _SFXMANAGER_INCLUDED_
#define _SFXMANAGER_INCLUDED_

#include "Prereqs.h"

#include "HelperFunctions.h"
#include "3dmath.h"
#include <DarkSDK.h>

class SFXManager
{
public:
	// subclass - SFX Object class- base class for all special effect objects.
	class SFXObject
	{
	public:
		SFXObject() {}

		virtual ~SFXObject() {}

		// main function to update the SFX object.
		virtual void _update( float timeStep ) = 0;

		// this function should return true if the object is done (should be deleted), otherwise return false.
		virtual bool _deleteme() = 0;
	};


	// get the singelton object
	static SFXManager& getSingleton();

	~SFXManager();

	// add a special effect object to the world.  once you give it an object, it will update it for you,
	// and automatically delete it when the object requests.
	void addSFXObject( SFXObject* obj );

	// update all objects.
	void updateAll( float timeStep );

private:
	SFXManager();

	typedef std::list<SFXManager::SFXObject*> SFXObjectList;
	typedef std::list<SFXManager::SFXObject*>::iterator SFXObjectListIterator;

	SFXObjectList mObjects;

};









#endif	// _SFXMANAGER_INCLUDED_