#include "SFX_Smoke.h"


SFX_Smoke::SFX_Smoke( float pps, int imgNum ) : SFXManager::SFXObject()
{
	mStillAdding = true;
	mPPS = pps;
	mTimeToNext = 0.0f;

	mImgNum = imgNum;
}


SFX_Smoke::~SFX_Smoke()
{
}


void SFX_Smoke::_update( float timeStep )
{
	// still adding?
	WorldInfo& info = WorldInfo::getSingleton();

	if (!info.mFlyingBall) { mStillAdding = false; }
	
	// make a new particle?
	mTimeToNext -= timeStep;

	if ((mTimeToNext < 0.0f) && (mStillAdding))
	{
		// make a new particle!
		dVector vel = info.mFlyingBall->getVelocity();
		vel = vel.Scale( 0.01f );
		SmokeParticle* part = new SmokeParticle( info.mFlyingBall->getPosition(), vel, 50.0f, 20.0f, 100.0f, 1.0f, mImgNum );
		mParticles.push_back( part );

		mTimeToNext = (1.0f / mPPS);
	}

	SmokeParticleList toDelete;

	//now go through and update all existing particles.
	for (SmokeParticleListIterator it = mParticles.begin(); it != mParticles.end(); it++ )
	{
		(*it)->update( timeStep );
		
		// this particle still alive?
		if ((*it)->mLife <= 0.0f)
		{
			toDelete.push_back( (*it) );
		}
	}

	// delete any particles to be deleted.
	for (SmokeParticleListIterator it = toDelete.begin(); it != toDelete.end(); it++)
	{
		SmokeParticle* part = (*it);

		mParticles.remove( part );

		delete part;
	}

}

bool SFX_Smoke::_deleteme()
{
	if (mStillAdding) { return false; }

	return (mParticles.size() == 0);
}


SFX_Smoke::SmokeParticle::SmokeParticle(dVector pos, dVector vel, float spin, float drag, float expansion, float life, int imgNum)
{
	mPos = pos;
	mVel = vel;
	mSpin = spin;
	mDrag = drag;
	mExpansion = expansion;
	mLife = life;

	mScale = 100.0f;

	// create the object.
	mObjNum = Helpers::getOpenObjNum();
	dbMakeObjectPlane( mObjNum, 1.0f, 1.0f );
	dbPositionObject( mObjNum, pos.m_x, pos.m_y, pos.m_z );
	dbTextureObject( mObjNum, imgNum );
	dbGhostObjectOn( mObjNum, 2 );
}

SFX_Smoke::SmokeParticle::~SmokeParticle()
{
	if (dbObjectExist( mObjNum ))
		dbDeleteObject( mObjNum );
}

void SFX_Smoke::SmokeParticle::update(float timeStep)
{
	mLife -= timeStep;

	if (mLife < 0.0f)
		return;

	dVector vel = mVel;
	vel = vel.Scale(mDrag * timeStep);
	mVel = mVel - vel;
	mSpin -= (mDrag * timeStep);
	
	mScale += mExpansion * timeStep;

	dVector add = mVel;
	add = add.Scale( timeStep );
	mPos += add;

	dbPositionObject( mObjNum, mPos.m_x, mPos.m_y, mPos.m_z );
	dbRollObjectLeft( mObjNum, mSpin * timeStep );
	dbScaleObject( mObjNum, mScale, mScale, mScale );

	// object fade.
	float fade = (mLife * 150.0f);
	dbFadeObject( mObjNum, fade );

}

