#include "ScoreBoard.h"

ScoreBoard::ScoreBoard( int sizex, int sizey, float charsizex, float charsizey, float charspacing, char*texImgFile, dVector position, dVector rot )
{
	mSizeX = sizex;
	mSizeY = sizey;

	mImgNum = Helpers::getOpenImageNum();
	dbLoadImage( texImgFile, mImgNum );

	// make the basic mesh for each character.
	// memblock represents 6 vertices at 36 bytes each + 12 byte header.
	int fvf = 338;
	int vertSize = 36;
	int numVerts = 6;
	int offset = 0;

	int memSize = (numVerts * vertSize) + 12;
	int mem = Helpers::getOpenMemblockNum();
	dbMakeMemblock( mem, memSize );

	dbWriteMemblockDWORD( mem, offset, fvf );	offset += 4;
	dbWriteMemblockDWORD( mem, offset, vertSize );	offset += 4;
	dbWriteMemblockDWORD( mem, offset, numVerts );	offset += 4;

	// create the memblock data.  by hand because it's so simple in this case.
	dVector pos;
	dVector norm;
	int diffuse;
	dVector uv;

	//vert 1.
	pos = dVector(-charsizex/2.0f, charsizey/2.0f, 0.0f);
	norm = dVector(0.0f, 0.0f, -1.0f);
	diffuse = dbRgb(255,255,255);
	uv = dVector( 0.0f, 0.0f, 0.0f );

	dbWriteMemblockFloat( mem, offset, pos.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_z );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_z );	offset += 4;
	dbWriteMemblockDWORD( mem, offset, diffuse );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_y );	offset += 4;

	//vert 2.
	pos = dVector(charsizex/2.0f, -charsizey/2.0f, 0.0f);
	norm = dVector(0.0f, 0.0f, -1.0f);
	diffuse = dbRgb(255,255,255);
	uv = dVector( 1.0f/16.0f, 1.0f/16.0f, 0.0f );

	dbWriteMemblockFloat( mem, offset, pos.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_z );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_z );	offset += 4;
	dbWriteMemblockDWORD( mem, offset, diffuse );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_y );	offset += 4;

	//vert 3.
	pos = dVector(-charsizex/2.0f, -charsizey/2.0f, 0.0f);
	norm = dVector(0.0f, 0.0f, -1.0f);
	diffuse = dbRgb(255,255,255);
	uv = dVector( 0.0f, 1.0f/16.0f, 0.0f );

	dbWriteMemblockFloat( mem, offset, pos.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_z );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_z );	offset += 4;
	dbWriteMemblockDWORD( mem, offset, diffuse );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_y );	offset += 4;

	//vert 4.
	pos = dVector(-charsizex/2.0f, charsizey/2.0f, 0.0f);
	norm = dVector(0.0f, 0.0f, -1.0f);
	diffuse = dbRgb(255,255,255);
	uv = dVector( 0.0f, 0.0f, 0.0f );

	dbWriteMemblockFloat( mem, offset, pos.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_z );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_z );	offset += 4;
	dbWriteMemblockDWORD( mem, offset, diffuse );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_y );	offset += 4;

	//vert 5.
	pos = dVector(charsizex/2.0f, charsizey/2.0f, 0.0f);
	norm = dVector(0.0f, 0.0f, -1.0f);
	diffuse = dbRgb(255,255,255);
	uv = dVector( 1.0f/16.0f, 0.0f, 0.0f );

	dbWriteMemblockFloat( mem, offset, pos.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_z );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_z );	offset += 4;
	dbWriteMemblockDWORD( mem, offset, diffuse );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_y );	offset += 4;

	//vert 6.
	pos = dVector(charsizex/2.0f, -charsizey/2.0f, 0.0f);
	norm = dVector(0.0f, 0.0f, -1.0f);
	diffuse = dbRgb(255,255,255);
	uv = dVector( 1.0f/16.0f, 1.0f/16.0f, 0.0f );

	dbWriteMemblockFloat( mem, offset, pos.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, pos.m_z );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_y );	offset += 4;
	dbWriteMemblockFloat( mem, offset, norm.m_z );	offset += 4;
	dbWriteMemblockDWORD( mem, offset, diffuse );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_x );	offset += 4;
	dbWriteMemblockFloat( mem, offset, uv.m_y );	offset += 4;


	// okay, the memblock is done.  let's make the main mesh from it.
	int mesh = Helpers::getOpenMeshNum();

	dbMakeMeshFromMemblock( mesh, mem );

	// now make a big object with lots of limbs based on this mesh.
	mLetters = new LetterInfo[ mSizeX * mSizeY ];
	
	dVector letteroffset(charsizex + charspacing, -charsizey - charspacing, 0.0f);

	// setup the rotation
	dMatrix mat = Helpers::makedMatrixFromEulerPos( rot, position );

	// loop through all letters, making new objects.
	for (int y=0; y<mSizeY; y++)
	{
		for (int x=0; x<mSizeX; x++)
		{
			int obj = (y*mSizeX)+x;
			mLetters[obj].mObjNum = Helpers::getOpenObjNum();

			dbMakeObject( mLetters[obj].mObjNum, mesh, 0 );
			
			dVector letterpos = dVector(letteroffset.m_x * x, letteroffset.m_y * y, 0.0f);
			letterpos = mat.TransformVector( letterpos );
			dbPositionObject( mLetters[obj].mObjNum, letterpos.m_x, letterpos.m_y, letterpos.m_z );
			dbRotateObject( mLetters[obj].mObjNum, rot.m_x, rot.m_y, rot.m_z );
			dbTextureObject( mLetters[obj].mObjNum, mImgNum );
			dbSetObjectLight( mLetters[obj].mObjNum, 0 );
		}
	}


	// cleanup
	dbDeleteMemblock( mem );
	dbDeleteMesh( mesh );

}

ScoreBoard::~ScoreBoard()
{
	// loop through all letters, making new objects.
	for (int y=0; y<mSizeY; y++)
	{
		for (int x=0; x<mSizeX; x++)
		{
			int obj = (y*mSizeX)+x;
			dbDeleteObject( mLetters[obj].mObjNum );
		}
	}

	dbDeleteImage( mImgNum );
}


void ScoreBoard::setRowText(int row, char* text )
{
	if ((row >= 0) && (row < mSizeY))
	{
		int textlength = dbLen( text );
		int textoff = 0;

		for (int x=0; x<mSizeX; x++)
		{
			// undo any scrolling done so far.
			LetterInfo& info = mLetters[ (row*mSizeX)+x ];

			dbScrollObjectTexture( info.mObjNum, -info.mXOffset, -info.mYOffset );
			
			char* thechar = dbChr(0);
			
			if (textlength >= (mSizeX-x))
				thechar = dbMid( text, ++textoff );

			int offset = dbAsc( thechar );

			int ycount = (int)offset / (int)16;
			info.mYOffset = (float)(ycount / 16.0f);
			info.mXOffset = (float)((offset-(ycount*16)) / 16.0f);

			dbScrollObjectTexture( info.mObjNum, info.mXOffset, info.mYOffset );

			thechar++;
		}
	}

}