#include "SoundManager.h"

SoundManager::Sound::Sound( std::string filename )
{
	mSoundNum = Helpers::getOpenSoundNum();

	dbLoadSound( (char*)(filename.c_str()), mSoundNum );
}


SoundManager::Sound::~Sound()
{
	dbDeleteSound( mSoundNum );
}


void SoundManager::Sound::play()
{
	dbPlaySound( mSoundNum );
}

bool SoundManager::Sound::isPlaying()
{
	return (dbSoundPlaying( mSoundNum ) == 1);
}


SoundManager::Music::Music( std::string filename )
{
	mMusicNum = Helpers::getOpenMusicNum();

	dbLoadMusic( (char*)(filename.c_str()), mMusicNum );
}

SoundManager::Music::~Music()
{
	dbDeleteMusic( mMusicNum );
}

void SoundManager::Music::play()
{
	dbPlayMusic( mMusicNum );
}

void SoundManager::Music::loopPlay()
{
	dbLoopMusic( mMusicNum );
}

bool SoundManager::Music::isPlaying()
{
	return (dbMusicPlaying( mMusicNum ) == 1);
}

void SoundManager::Music::stop()
{
	dbStopMusic( mMusicNum );
}



SoundManager& SoundManager::getSingleton()
{
	static SoundManager instance;
	return instance;
}


SoundManager::Sound* SoundManager::addSound( std::string name, std::string filename )
{
	// does a sound with this name exist?
	SoundMapIterator it = mSounds.find( name );

	if (it != mSounds.end() )
		return NULL;

	// okay, sound doesn't exist, so create it.
	Sound* newsound = new Sound( filename );

	mSounds[ name ] = newsound;

	return newsound;
}

SoundManager::Sound* SoundManager::getSound( std::string name )
{
	SoundMapIterator it = mSounds.find( name );

	if (it != mSounds.end())
		return it->second;

	return NULL;
}

void SoundManager::playSound( std::string name )
{
	// sound exists?
	SoundMapIterator it = mSounds.find( name );

	if (it != mSounds.end())
	{
		it->second->play();
	}
}

void SoundManager::removeSound( std::string name )
{
	// sound exists?
	SoundMapIterator it = mSounds.find( name );

	if (it != mSounds.end())
	{
		Sound* kill = it->second;

		delete kill;

		mSounds.erase( name );
	}
}


SoundManager::Music* SoundManager::addMusic( std::string name, std::string filename )
{
	// does a sound with this name exist?
	MusicMapIterator it = mMusic.find( name );

	if (it != mMusic.end() )
		return NULL;

	// okay, sound doesn't exist, so create it.
	Music* newmusic = new Music( filename );

	mMusic[ name ] = newmusic;

	return newmusic;
}


SoundManager::Music* SoundManager::getMusic( std::string name )
{
	MusicMapIterator it = mMusic.find( name );

	if (it != mMusic.end())
		return it->second;

	return NULL;
}

void SoundManager::playMusic( std::string name )
{
	// sound exists?
	MusicMapIterator it = mMusic.find( name );

	if (it != mMusic.end())
	{
		it->second->play();
	}
}

void SoundManager::loopMusic( std::string name )
{
	// sound exists?
	MusicMapIterator it = mMusic.find( name );

	if (it != mMusic.end())
	{
		it->second->loopPlay();
	}
}

void SoundManager::removeMusic( std::string name )
{
	// sound exists?
	MusicMapIterator it = mMusic.find( name );

	if (it != mMusic.end())
	{
		Music* kill = it->second;

		delete kill;

		mMusic.erase( name );
	}
}


SoundManager::SoundManager()
{
}


SoundManager::~SoundManager()
{
	for(SoundMapIterator it=mSounds.begin(); it != mSounds.end(); it++)
	{
		Sound* tokill = it->second;

		delete tokill;
	}

	mSounds.clear();


	for(MusicMapIterator it=mMusic.begin(); it != mMusic.end(); it++)
	{
		Music* tokill = it->second;

		delete tokill;
	}

	mMusic.clear();
}