/*
	ConnectMore project

	by Walaber

	CLASS: WorldInfo - various global variables enclosed in a singleton class.
*/

#ifndef _WORLDINFO_INCLUDED_
#define _WORLDINFO_INCLUDED_

#include "Prereqs.h"
#include "BasicBall.h"
#include "MenuSystem.h"


class WorldInfo
{
public:
	static WorldInfo& getSingleton()
	{
		static WorldInfo instance;
		return instance;
	}

	~WorldInfo() {}

	// main game state!
	GameState mState;

	// current level
	unsigned int mLevel;

	// mouse stuff.
	int mMouseX;
	int mMouseY;

	int mMouseMoveX;
	int mMouseMoveY;

	
	NewtonWorld* mWorld;
	int mMatBall;
	int mMatFrame;

	// root ball (used for object cloning)
	int mRootBallObjNum;

	// root image for smoke particles.
	int mRootSmoke1ImgNum;
	int mRootSmoke2ImgNum;

	// image array for bomb images.
	int mBombImgNum[16];

	// next ball information
	float mTimeToNextBall;
	int mCurrentSpout;
	std::vector<Spout> mSpouts;

	// color information
	typedef std::map<Colors, ColorInfo> ColorInfoMap;
	typedef std::map<Colors, ColorInfo>::iterator ColorInfoMapIterator;
	ColorInfoMap mColors;

	// SCORING variables...
	unsigned long mScore;
	unsigned short mComboCount;
	unsigned short mMaxCombo;

	// list of balls to delete each loop.
	BasicBall::BallContactMap mToDelete;

	// pointer to flying ball.
	BasicBall* mFlyingBall;

	// special pointer for wild ball reaction.
	BasicBall* mKillAllLikeMe;

	// special pointer for bomb application.
	BasicBall* mBombBall;

	// menu manager pointer.
	MenuManager* mMenuMgr;


	// high scores!
	std::vector<HiScore> mHiScores;

	// sprite for fading screen.
	int mFadeSpriteImg;
	int mFadeSprite;


private:
	WorldInfo();

};

#endif	// _WORLDINFO_INCLUDED_